@@ -107,7 +107,7 @@ def get_debug_queries():
     one expected on errors or in unittesting.  If you don't want to enable
     the DEBUG mode for your unittests you can also enable the query
     recording by setting the ``'SQLALCHEMY_RECORD_QUERIES'`` config variable
-    to `True`.
+    to `True`.  This is automatically enablde if Flask is in testing mode.
 
     The value returned will be a list of named tuples with the following
     attributes:
@@ -286,6 +286,15 @@ class _QueryProperty(object):
             return None
 
 
+def _record_queries(app):
+    if app.debug:
+        return True
+    rq = app.config['SQLALCHEMY_RECORD_QUERIES']
+    if rq is not None:
+        return rq
+    return bool(app.config.get('TESTING'))
+
+
 class _EngineConnector(object):
 
     def __init__(self, sa, app):
@@ -303,9 +312,9 @@ class _EngineConnector(object):
                 return self._engine
             info = make_url(uri)
             options = {'convert_unicode': True}
-            self._sa.apply_driver_hacks(info, options)
-            if self._app.debug or \
-               self._app.config['SQLALCHEMY_RECORD_QUERIES']:
+            self._sa.apply_pool_defaults(self._app, options)
+            self._sa.apply_driver_hacks(self._app, info, options)
+            if _record_queries(self._app):
                 options['proxy'] = _ConnectionDebugProxy(self._app.import_name)
             if echo:
                 options['echo'] = True
@@ -357,7 +366,9 @@ class SQLAlchemy(object):
     will probe the library for native unicode support.  If it detects
     unicode it will let the library handle that, otherwise do that itself.
     Sometimes this detection can fail in which case you might want to set
-    `use_native_unicode` to `False`.
+    `use_native_unicode` (or the ``SQLALCHEMY_NATIVE_UNICODE`` configuration
+    key) to `False`.  Note that the configuration key overrides the
+    value you pass to the constructor.
 
     Additionally this class also provides access to all the SQLAlchemy
     functions from the :mod:`sqlalchemy` and :mod:`sqlalchemy.orm` modules.
@@ -385,6 +396,11 @@ class SQLAlchemy(object):
 
         _include_sqlalchemy(self)
 
+    @property
+    def metadata(self):
+        """Returns the metadata"""
+        return self.Model.metadata
+
     def init_app(self, app):
         """This callback can be used to initialize an application for the
         use with this database setup.  Never use a database in the context
@@ -392,18 +408,37 @@ class SQLAlchemy(object):
         leak.
         """
         app.config.setdefault('SQLALCHEMY_DATABASE_URI', 'sqlite://')
+        app.config.setdefault('SQLALCHEMY_NATIVE_UNICODE', None)
         app.config.setdefault('SQLALCHEMY_ECHO', False)
-        app.config.setdefault('SQLALCHEMY_RECORD_QUERIES', False)
+        app.config.setdefault('SQLALCHEMY_RECORD_QUERIES', None)
+        app.config.setdefault('SQLALCHEMY_POOL_SIZE', None)
+        app.config.setdefault('SQLALCHEMY_POOL_TIMEOUT', None)
+        app.config.setdefault('SQLALCHEMY_POOL_RECYCLE', None)
 
         @app.after_request
         def shutdown_session(response):
             self.session.remove()
             return response
 
-    def apply_driver_hacks(self, info, options):
+    def apply_pool_defaults(self, app, options):
+        def _setdefault(optionkey, configkey):
+            value = app.config[configkey]
+            if value is not None:
+                options[optionkey] = value
+        _setdefault('pool_size', 'SQLALCHEMY_POOL_SIZE')
+        _setdefault('pool_timeout', 'SQLALCHEMY_POOL_TIMEOUT')
+        _setdefault('pool_recycle', 'SQLALCHEMY_POOL_RECYCLE')
+
+    def apply_driver_hacks(self, app, info, options):
         if info.drivername == 'mysql':
             info.query.setdefault('charset', 'utf8')
-        if not self.use_native_unicode:
+            options.setdefault('pool_size', 10)
+            options.setdefault('pool_recycle', 7200)
+
+        unu = app.config['SQLALCHEMY_NATIVE_UNICODE']
+        if unu is None:
+            unu = self.use_native_unicode
+        if not unu:
             options['use_native_unicode'] = False
 
     @property
@@ -444,7 +479,14 @@ class SQLAlchemy(object):
         self.Model.metadata.reflect(bind=self.engine)
 
     def __repr__(self):
+        app = None
+        if self.app is not None:
+            app = self.app
+        else:
+            ctx = _request_ctx_stack.top
+            if ctx is not None:
+                app = ctx.app
         return '<%s engine=%r>' % (
             self.__class__.__name__,
-            self.app.config['SQLALCHEMY_DATABASE_URI']
+            app and self.app.config['SQLALCHEMY_DATABASE_URI'] or None
         )
